@@ -57,6 +57,8 @@ static int short_docs_mode = FALSE;
 static int long_docs_mode = FALSE;
 static int schema_name_mode = FALSE;
 static int associate_schema_mode = FALSE;
+static int default_source_mode = FALSE;
+static int recursive_unset_mode = FALSE;
 
 struct poptOption options[] = {
   { 
@@ -105,6 +107,15 @@ struct poptOption options[] = {
     N_("Unset the keys on the command line"),
     NULL
   },
+  {
+    "recursive-unset",
+    '\0',
+    POPT_ARG_NONE,
+    &recursive_unset_mode,
+    0, 
+    N_("Recursively unset all keys at or below the key/directory names on the command line"),
+    NULL
+  },
   { 
     "all-entries",
     'a',
@@ -321,6 +332,15 @@ struct poptOption options[] = {
     N_("Specify the schema name followed by the key to apply the schema name to"),
     NULL
   },
+  {
+    "get-default-source",
+    '\0',
+    POPT_ARG_NONE,
+    &default_source_mode,
+    0,
+    N_("Get the name of the default source"),
+    NULL
+  },
   {
     NULL,
     '\0',
@@ -345,12 +365,14 @@ static int do_set(GConfEngine* conf, const gchar** args);
 static int do_set_schema(GConfEngine* conf, const gchar** args);
 static int do_all_entries(GConfEngine* conf, const gchar** args);
 static int do_unset(GConfEngine* conf, const gchar** args);
+static int do_recursive_unset (GConfEngine* conf, const gchar** args);
 static int do_all_subdirs(GConfEngine* conf, const gchar** args);
 static int do_load_schema_file(GConfEngine* conf, const gchar* file);
 static int do_short_docs (GConfEngine *conf, const gchar **args);
 static int do_long_docs (GConfEngine *conf, const gchar **args);
 static int do_get_schema_name (GConfEngine *conf, const gchar **args);
 static int do_associate_schema (GConfEngine *conf, const gchar **args);
+static int do_get_default_source (GConfEngine *conf, const gchar **args);
 
 int 
 main (int argc, char** argv)
@@ -506,6 +528,7 @@ main (int argc, char** argv)
       return 1;
     }
 
+  /* FIXME not checking that --recursive-unset is used alone */
   
   if (use_local_source && config_source == NULL)
     {
@@ -530,6 +553,16 @@ main (int argc, char** argv)
       else 
         return 2;
     }
+
+  /* Before creating engine */
+  if (default_source_mode)
+    {
+      const gchar** args = poptGetArgs(ctx);
+      if (do_get_default_source (conf, args)  == 1)
+        return 1;
+      else
+        return 0;
+    }
   
   if (makefile_install_mode)
     {
@@ -549,19 +582,11 @@ main (int argc, char** argv)
           /* Properly set, but set to nothing (use default source) */
           config_source = NULL;
         }
-      
-      if (gconf_ping_daemon())
-        {
-          /* Eventually you should be able to run gconfd as long as
-             you're installing to a different database from the one
-             it's using, but I don't trust the locking right now. */
-          fprintf(stderr, _("Shouldn't run gconfd while installing new schema files.\nUse gconftool --shutdown to shut down the daemon, most safely while no applications are running\n(though things theoretically work if apps are running).\n"));
-          return 1;
-        }
 
-      /* Race condition! gconfd could start up. But we do have locking
-         as a second line of defense. */
       use_local_source = TRUE;
+
+      /* shut down daemon, this is a race condition, but will usually work. */
+      gconf_shutdown_daemon (NULL);
     }
   
   if (config_source == NULL)
@@ -744,6 +769,17 @@ main (int argc, char** argv)
         }
     }
 
+  if (recursive_unset_mode)
+    {
+      const gchar** args = poptGetArgs(ctx);
+
+      if (do_recursive_unset(conf, args) == 1)
+        {
+          gconf_engine_unref(conf);
+          return 1;
+        }
+    }
+  
   if (all_subdirs_mode)
     {
       const gchar** args = poptGetArgs(ctx);
@@ -1527,6 +1563,112 @@ do_unset(GConfEngine* conf, const gchar** args)
   return 0;
 }
 
+static void
+recursive_unset_helper (GConfEngine *conf,
+                        const char  *key)
+{
+  GError* err = NULL;
+  GSList* subdirs;
+  GSList* entries;
+  GSList* tmp;
+
+  err = NULL;
+  
+  subdirs = gconf_engine_all_dirs (conf, key, &err);
+          
+  if (subdirs != NULL)
+    {
+      tmp = subdirs;
+
+      while (tmp != NULL)
+        {
+          gchar* s = tmp->data;
+
+          recursive_unset_helper (conf, s);
+          
+          g_free (s);
+
+          tmp = g_slist_next (tmp);
+        }
+
+      g_slist_free (subdirs);
+    }
+  else
+    {
+      if (err != NULL)
+        {
+          fprintf (stderr, _("Error listing subdirs of '%s': %s\n"),
+                   key, err->message);
+          g_error_free(err);
+          err = NULL;
+        }
+    }
+
+  entries = gconf_engine_all_entries (conf, key, &err);
+          
+  if (err != NULL)
+    {
+      fprintf (stderr, _("Failure listing entries in '%s': %s\n"),
+               key, err->message);
+      g_error_free(err);
+      err = NULL;
+    }
+
+  if (entries != NULL)
+    {
+      tmp = entries;
+
+      while (tmp != NULL)
+        {
+          GConfEntry* entry = tmp->data;
+          
+          gconf_engine_unset (conf,
+                              gconf_entry_get_key (entry),
+                              &err);
+          if (err != NULL)
+            {
+              fprintf (stderr, _("Error unsetting '%s': %s\n"),
+                       gconf_entry_get_key (entry), err->message);
+              g_error_free (err);
+              err = NULL;
+            }
+          
+          gconf_entry_free (entry);
+
+          tmp = g_slist_next (tmp);
+        }
+
+      g_slist_free (entries);
+    }
+
+  gconf_engine_unset (conf, key, &err);
+  if (err != NULL)
+    {
+      fprintf (stderr, _("Error unsetting '%s': %s\n"),
+               key, err->message);
+      g_error_free (err);
+      err = NULL;
+    }
+}
+
+static int
+do_recursive_unset (GConfEngine* conf, const gchar** args)
+{
+  if (args == NULL)
+    {
+      fprintf(stderr, _("Must specify one or more keys to recursively unset.\n"));
+      return 1;
+    }  
+      
+  while (*args)
+    {
+      recursive_unset_helper (conf, *args);
+      
+      ++args;
+    }
+
+  return 0;
+}
 
 static int
 do_all_subdirs(GConfEngine* conf, const gchar** args)
@@ -2500,3 +2642,36 @@ do_break_directory(GConfEngine* conf, const gchar** args)
 
   return 0;
 }
+
+static int
+do_get_default_source(GConfEngine* conf, const gchar** args)
+{
+  gchar *source;
+  gchar buf[512];
+  FILE *f;
+
+  /* Try with $sysconfdir/gconf/schema-install-source */
+  f = fopen(GCONF_ETCDIR"/schema-install-source", "r");
+
+  if (f != NULL)
+    {
+      source = fgets(buf, 512, f);
+      fclose(f);  
+      if (source)
+	{
+	  g_strchomp(source);
+	  if (*source != '\0')
+	    {
+	      printf("%s\n", source);
+	      return 0;
+	    }
+	}
+    }
+
+  /* Use default database */
+  source = g_strconcat("xml::", GCONF_ETCDIR, "/gconf.xml.defaults", NULL);
+  printf("%s\n", source);
+  g_free(source);
+
+  return 0;
+}
